@@ -1,19 +1,6 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2019 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-# This class contains the basic functionality needed to run any interpreter
-# or an interpreter-based tool.
 from __future__ import annotations
 
 from ..mesonlib import MesonException, OptionKey
@@ -123,7 +110,7 @@ def cmake_get_generator_args(env: 'Environment') -> T.List[str]:
     return ['-G', backend_generator_map[backend_name]]
 
 def cmake_defines_to_args(raw: T.List[T.Dict[str, TYPE_var]], permissive: bool = False) -> T.List[str]:
-    res = []  # type: T.List[str]
+    res: T.List[str] = []
 
     for i in raw:
         for key, val in i.items():
@@ -144,7 +131,7 @@ def cmake_defines_to_args(raw: T.List[T.Dict[str, TYPE_var]], permissive: bool =
 
 # TODO: this function will become obsolete once the `cmake_args` kwarg is dropped
 def check_cmake_args(args: T.List[str]) -> T.List[str]:
-    res = []  # type: T.List[str]
+    res: T.List[str] = []
     dis = ['-D' + x for x in blacklist_cmake_defs]
     assert dis  # Ensure that dis is not empty.
     for i in args:
@@ -166,14 +153,14 @@ class CMakeInclude:
 
 class CMakeFileGroup:
     def __init__(self, data: T.Dict[str, T.Any]) -> None:
-        self.defines = data.get('defines', '')                       # type: str
-        self.flags = _flags_to_list(data.get('compileFlags', ''))  # type: T.List[str]
-        self.is_generated = data.get('isGenerated', False)                # type: bool
-        self.language = data.get('language', 'C')                     # type: str
-        self.sources = [Path(x) for x in data.get('sources', [])]    # type: T.List[Path]
+        self.defines: str = data.get('defines', '')
+        self.flags = _flags_to_list(data.get('compileFlags', ''))
+        self.is_generated: bool = data.get('isGenerated', False)
+        self.language: str = data.get('language', 'C')
+        self.sources = [Path(x) for x in data.get('sources', [])]
 
         # Fix the include directories
-        self.includes = []  # type: T.List[CMakeInclude]
+        self.includes: T.List[CMakeInclude] = []
         for i in data.get('includePath', []):
             if isinstance(i, dict) and 'path' in i:
                 isSystem = i.get('isSystem', False)
@@ -196,21 +183,21 @@ class CMakeFileGroup:
 
 class CMakeTarget:
     def __init__(self, data: T.Dict[str, T.Any]) -> None:
-        self.artifacts = [Path(x) for x in data.get('artifacts', [])]               # type: T.List[Path]
-        self.src_dir = Path(data.get('sourceDirectory', ''))                        # type: Path
-        self.build_dir = Path(data.get('buildDirectory', ''))                       # type: Path
-        self.name = data.get('name', '')                                            # type: str
-        self.full_name = data.get('fullName', '')                                   # type: str
-        self.install = data.get('hasInstallRule', False)                            # type: bool
-        self.install_paths = [Path(x) for x in set(data.get('installPaths', []))]   # type: T.List[Path]
-        self.link_lang = data.get('linkerLanguage', '')                             # type: str
-        self.link_libraries = _flags_to_list(data.get('linkLibraries', ''))         # type: T.List[str]
-        self.link_flags = _flags_to_list(data.get('linkFlags', ''))                 # type: T.List[str]
-        self.link_lang_flags = _flags_to_list(data.get('linkLanguageFlags', ''))    # type: T.List[str]
-        # self.link_path = Path(data.get('linkPath', ''))                             # type: Path
-        self.type = data.get('type', 'EXECUTABLE')                                  # type: str
-        # self.is_generator_provided = data.get('isGeneratorProvided', False)         # type: bool
-        self.files = []                                                             # type: T.List[CMakeFileGroup]
+        self.artifacts = [Path(x) for x in data.get('artifacts', [])]
+        self.src_dir = Path(data.get('sourceDirectory', ''))
+        self.build_dir = Path(data.get('buildDirectory', ''))
+        self.name: str = data.get('name', '')
+        self.full_name: str = data.get('fullName', '')
+        self.install: bool = data.get('hasInstallRule', False)
+        self.install_paths = [Path(x) for x in set(data.get('installPaths', []))]
+        self.link_lang: str = data.get('linkerLanguage', '')
+        self.link_libraries = _flags_to_list(data.get('linkLibraries', ''))
+        self.link_flags = _flags_to_list(data.get('linkFlags', ''))
+        self.link_lang_flags = _flags_to_list(data.get('linkLanguageFlags', ''))
+        # self.link_path = Path(data.get('linkPath', ''))
+        self.type: str = data.get('type', 'EXECUTABLE')
+        # self.is_generator_provided: bool = data.get('isGeneratorProvided', False)
+        self.files: T.List[CMakeFileGroup] = []
 
         for i in data.get('fileGroups', []):
             self.files += [CMakeFileGroup(i)]
@@ -237,10 +224,10 @@ class CMakeTarget:
 
 class CMakeProject:
     def __init__(self, data: T.Dict[str, T.Any]) -> None:
-        self.src_dir = Path(data.get('sourceDirectory', ''))    # type: Path
-        self.build_dir = Path(data.get('buildDirectory', ''))   # type: Path
-        self.name = data.get('name', '')                        # type: str
-        self.targets = []                                       # type: T.List[CMakeTarget]
+        self.src_dir = Path(data.get('sourceDirectory', ''))
+        self.build_dir = Path(data.get('buildDirectory', ''))
+        self.name: str = data.get('name', '')
+        self.targets: T.List[CMakeTarget] = []
 
         for i in data.get('targets', []):
             self.targets += [CMakeTarget(i)]
@@ -256,8 +243,8 @@ class CMakeProject:
 
 class CMakeConfiguration:
     def __init__(self, data: T.Dict[str, T.Any]) -> None:
-        self.name = data.get('name', '')        # type: str
-        self.projects = []                      # type: T.List[CMakeProject]
+        self.name: str = data.get('name', '')
+        self.projects: T.List[CMakeProject] = []
         for i in data.get('projects', []):
             self.projects += [CMakeProject(i)]
 
@@ -270,9 +257,9 @@ class CMakeConfiguration:
 
 class SingleTargetOptions:
     def __init__(self) -> None:
-        self.opts = {}       # type: T.Dict[str, str]
-        self.lang_args = {}  # type: T.Dict[str, T.List[str]]
-        self.link_args = []  # type: T.List[str]
+        self.opts: T.Dict[str, str] = {}
+        self.lang_args: T.Dict[str, T.List[str]] = {}
+        self.link_args: T.List[str] = []
         self.install = 'preserve'
 
     def set_opt(self, opt: str, val: str) -> None:
@@ -290,7 +277,7 @@ class SingleTargetOptions:
         self.install = 'true' if install else 'false'
 
     def get_override_options(self, initial: T.List[str]) -> T.List[str]:
-        res = []  # type: T.List[str]
+        res: T.List[str] = []
         for i in initial:
             opt = i[:i.find('=')]
             if opt not in self.opts:
@@ -312,7 +299,7 @@ class SingleTargetOptions:
 class TargetOptions:
     def __init__(self) -> None:
         self.global_options = SingleTargetOptions()
-        self.target_options = {}  # type: T.Dict[str, SingleTargetOptions]
+        self.target_options: T.Dict[str, SingleTargetOptions] = {}
 
     def __getitem__(self, tgt: str) -> SingleTargetOptions:
         if tgt not in self.target_options:
